#------------------------------------------------------------------------------
#
# Project: prism view server
# Authors: Fabian Schindler <fabian.schindler@eox.at>
#
#------------------------------------------------------------------------------
# Copyright (C) 2020 EOX IT Services GmbH <https://eox.at>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies of this Software or works derived from this Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#-----------------------------------------------------------------------------

import io

from lxml import etree
import dateutil.parser

from .util import pairwise


class BrowseReportParserError(ValueError):
    pass


NS_REP = 'http://ngeo.eo.esa.int/ngEO/browseReport/1.0'
NS_REP_OLD = 'http://ngeo.eo.esa.int/schema/browseReport'
NS_BSI = 'http://ngeo.eo.esa.int/schema/browse/ingestion'
NS_TPZ = 'http://www.telespazio.com/CSCDA/CDD/PDAS'


def rep(tag):
    return f'{{{NS_REP}}}{tag}'


def rep_old(tag):
    return f'{{{NS_REP_OLD}}}{tag}'


def bsi(tag):
    return f'{{{NS_BSI}}}{tag}'


def tpz(tag):
    return f'{{{NS_TPZ}}}{tag}'


ALLOWED_ROOT_TAGS = {rep('browseReport'), rep_old('browseReport'), bsi('ingestBrowse'), tpz('OnLineArchiveItem')}


def parse_browse_report(input_file):
    """
        :returns: list of browses
    """
    if isinstance(input_file, bytes):
        input_file = io.BytesIO(input_file)
    try:
        tree = etree.parse(input_file)
    except etree.XMLSyntaxError as e:
        raise BrowseReportParserError('Failed to parse XML.') from e
    root = tree.getroot()

    if not root.tag in ALLOWED_ROOT_TAGS:
        raise BrowseReportParserError(
            'Document is not a browse report or an ingest browse instruction.'
        )

    if root.tag == rep_old('browseReport'):
        used_rep = rep_old
    elif root.tag == tpz('OnLineArchiveItem'):
        if root.findtext(tpz('status')) in ['added', 'updated']:
            path_el = root.find(tpz('path')).findtext(tpz('URL'))
            obs_path = path_el.replace('\n', '').replace('/', '', 1)
            return obs_path
        raise BrowseReportParserError('Status not in "added" or "updated", aborting.')
    else:
        used_rep = rep

    return {
        'responsible_org_name': root.findtext(used_rep('responsibleOrgName')),
        'date_time': dateutil.parser.parse(root.findtext(used_rep('dateTime'))),
        'browse_type': root.findtext(used_rep('browseType')),
        'browses': [
            parse_browse(elem, used_rep)
            for elem in root.iterfind(used_rep('browse'))
        ],
    }


def parse_browse(elem, used_rep):
    browse = {
        'type': '',
        'browse_identifier': elem.findtext(used_rep('browseIdentifier')),
        'filename': elem.findtext(used_rep('fileName')),
        'image_type': elem.findtext(used_rep('imageType')),
        'reference_system_identifier': elem.findtext(
            used_rep('referenceSystemIdentifier')
        ),
        'start_time': dateutil.parser.parse(elem.findtext(used_rep('startTime'))),
        'end_time': dateutil.parser.parse(elem.findtext(used_rep('endTime'))),
    }

    rectified_elem = elem.find(used_rep('rectifiedBrowse'))
    footprint_elem = elem.find(used_rep('footprint'))
    geotiff_elem = elem.find(used_rep('modelInGeotiff'))
    regular_grid_browse = elem.find(used_rep('regularGrid'))

    if rectified_elem is not None:
        browse['type'] = 'rectified_browse'
        browse['rectified'] = {
            'coord_list': [
                (float(x), float(y))
                for x, y in pairwise(
                    rectified_elem.findtext(used_rep('coordList')).split()
                )
            ],
        }

    elif footprint_elem is not None:
        browse['type'] = 'footprint_browse'
        browse['footprint'] = {
            'col_row_list': [
                (int(x), int(y))
                for x, y in pairwise(
                    footprint_elem.findtext(used_rep('colRowList')).split()
                )
            ],
            'coord_list': [
                (float(x), float(y))
                for x, y in pairwise(
                    footprint_elem.findtext(used_rep('coordList')).split()
                )
            ],
        }

    elif geotiff_elem is not None:
        browse['type'] = 'model_in_geotiff_browse'

    elif regular_grid_browse is not None:
        browse['type'] = 'regular_grid_browse'
        browse['regular_grid'] = {
            'col_node_number': int(regular_grid_browse.findtext(used_rep('colNodeNumber'))),
            'row_node_number': int(regular_grid_browse.findtext(used_rep('rowNodeNumber'))),
            'col_step': float(regular_grid_browse.findtext(used_rep('colStep'))),
            'row_step': float(regular_grid_browse.findtext(used_rep('rowStep'))),
            'coord_lists': [
                [
                    (float(x), float(y))
                    for x, y in pairwise(elem.text.split())
                ] for elem in regular_grid_browse.iterfind(used_rep('coordList'))
            ]
        }

    return browse
